@@ -141,7 +141,7 @@ dh:
             "dh": {1: 2},
         }
     )
-    df, is_valid, errors = v.validate(df)
+    df, is_valid, errors = v.validate_dataframe(df)
     assert is_valid is False
     expected_errors = {("da", 0): ["None is not of type 'integer'"]}
     assert errors == expected_errors
@@ -210,7 +210,7 @@ def test_VData_00():
         ("life_nb", 1): ["'hg' is not of type 'integer'"],
         ("life_nb", 2): ["15 is greater than the maximum of 4"],
     }
-    df, is_valid, errors = data.validate_tab("test")
+    df, is_valid, errors = data._validate_tab("test")
     assert is_valid is False
     assert errors == expected_report
 
@@ -242,7 +242,7 @@ def test_IVData_00(datadir):
     data.read_schema(os.path.join(indir, "test_00.schema.yaml"))
     for tab in ("names", "cars", "empty"):
         print('checking "%s"' % tab, end="... ")
-        df, is_ok, errors = data.validate_tab(tab)
+        df, is_ok, errors = data._validate_tab(tab)
         try:
             assert is_ok is True
         except:
@@ -266,7 +266,7 @@ def test_IVData_00(datadir):
         data_new.read_schema(os.path.join(indir, "test_00.schema.yaml"))
         for tab in data._data.keys():
             print('checking "%s"' % tab, end="... ")
-            df, is_ok, errors = data_new.validate_tab(tab)
+            df, is_ok, errors = data_new._validate_tab(tab)
             try:
                 assert is_ok is True
             except:
@@ -316,3 +316,56 @@ def test_IVData_03(datadir):
     }
 
     assert ret == expected_errors
+
+
+def test_IVData_04(datadir):
+    """check units"""
+    indir, outdir = datadir
+    data = IVData()
+    data.read(os.path.join(indir, "test_03_units.ini"))
+    # -------------------------------------------------------------------------
+    # read and check schema
+    data.read_schema(os.path.join(indir, "test_03_units.schema.yaml"), debug=True)
+    expected_schema = {
+        "$schema": "http://json-schema.org/draft-07/schema#",
+        "properties": {
+            "Volume": {
+                "items": {
+                    "anyOf": [{"type": "number"}, {"type": "null"}],
+                    "default": 0,
+                },
+                "type": "array",
+                "uniqueItems": False,
+                "units": "m^3",
+            },
+            "distA": {
+                "items": {"type": "number"},
+                "type": "array",
+                "uniqueItems": False,
+                "units": "m",
+            },
+            "distB": {
+                "items": {
+                    "anyOf": [{"type": "number"}, {"type": "null"}],
+                    "default": 0,
+                },
+                "type": "array",
+                "uniqueItems": False,
+                "units": "m",
+            },
+            "id": {"items": {"type": "integer"}, "type": "array", "uniqueItems": True},
+        },
+        "required": ["id", "distA"],
+        "type": "object",
+    }
+    assert data._schemas["geom"].build() == expected_schema
+    ret = data.validate()
+    expected_geom = pd.DataFrame(
+        {
+            "Volume": {1: 1.0, 2: 2.0},
+            "distA": {1: 0.001, 2: 0.002},
+            "distB": {1: 14.3, 2: 15.0},
+            "id": {1: 1.5064562386943998, 2: 0.8183568665087998},
+        }
+    )
+    assert pd.testing.assert_frame_equal(data._data['geom'], expected_geom) is None
